@@ -1,10 +1,10 @@
-"""Variable definitions"""
-# pylint: disable=too-many-lines
+"""Variable definitions."""
+
 from __future__ import annotations
 
-from collections.abc import Callable
+from collections.abc import Callable, Iterable
 from enum import Enum
-from typing import Iterable, Literal, NamedTuple
+from typing import Literal, NamedTuple
 
 import netCDF4
 
@@ -18,12 +18,16 @@ class Product(Enum):
     MODEL = "model"
     WEATHER_STATION = "weather-station"
     DOPPLER_LIDAR = "doppler-lidar"
+    DOPPLER_LIDAR_WIND = "doppler-lidar-wind"
     RAIN_RADAR = "rain-radar"
+    RAIN_GAUGE = "rain-gauge"
     # Level 1c
     CATEGORIZE = "categorize"
     MWR_L1C = "mwr-l1c"
+    CATEGORIZE_VOODOO = "categorize-voodoo"
     # Level 2
     CLASSIFICATION = "classification"
+    CLASSIFICATION_VOODOO = "classification-voodoo"
     IWC = "iwc"
     LWC = "lwc"
     DER = "der"
@@ -31,10 +35,12 @@ class Product(Enum):
     DRIZZLE = "drizzle"
     MWR_SINGLE = "mwr-single"
     MWR_MULTI = "mwr-multi"
+    EPSILON_LIDAR = "epsilon-lidar"
     # Level 3
     L3_CF = "l3-cf"
     L3_IWC = "l3-iwc"
     L3_LWC = "l3-lwc"
+    CPR = "cpr-simulation"
 
     @classmethod
     def all(cls) -> set[Product]:
@@ -51,10 +57,14 @@ LEVELS: dict[Product, Level] = {
     Product.MODEL: "1b",
     Product.WEATHER_STATION: "1b",
     Product.DOPPLER_LIDAR: "1b",
+    Product.RAIN_GAUGE: "1b",
+    Product.DOPPLER_LIDAR_WIND: "1c",
     Product.RAIN_RADAR: "1b",
     Product.CATEGORIZE: "1c",
+    Product.CATEGORIZE_VOODOO: "1c",
     Product.MWR_L1C: "1c",
     Product.CLASSIFICATION: "2",
+    Product.CLASSIFICATION_VOODOO: "2",
     Product.IWC: "2",
     Product.LWC: "2",
     Product.DER: "2",
@@ -62,9 +72,11 @@ LEVELS: dict[Product, Level] = {
     Product.DRIZZLE: "2",
     Product.MWR_SINGLE: "2",
     Product.MWR_MULTI: "2",
+    Product.EPSILON_LIDAR: "2",
     Product.L3_CF: "3",
     Product.L3_IWC: "3",
     Product.L3_LWC: "3",
+    Product.CPR: "3",
 }
 
 
@@ -73,10 +85,12 @@ class Dtype(str, Enum):
     DOUBLE = "float64"
     INT = "int32"
     SHORT = "int16"
+    BYTE = "int8"
 
 
 class Variable(NamedTuple):
     long_name: str
+    comment: str | None = None
     units: str | Callable[[netCDF4.Dataset], str] | None = "1"
     dtype: str = Dtype.FLOAT
     standard_name: str | None = None
@@ -132,38 +146,41 @@ VARIABLES = {
         standard_name="equivalent_reflectivity_factor",
         required=[Product.DISDROMETER],
     ),
-    "visibility": Variable(
-        long_name="Visibility range in precipitation after MOR",
-        units="m",
-        standard_name="visibility_in_air",
-        dtype=Dtype.INT,
-        required=[Product.DISDROMETER],
-    ),
     "n_particles": Variable(
         long_name="Number of particles in time interval",
         dtype=Dtype.INT,
-        required=[Product.DISDROMETER],
     ),
-    # -----------------------------------------
-    # Required in WEATHER STATION Level 1b file
-    # -----------------------------------------
+    # --------------------------------------------
+    # Recommended in WEATHER STATION Level 1b file
+    # --------------------------------------------
     "air_temperature": Variable(
         long_name="Air temperature",
         standard_name="air_temperature",
         units="K",
-        required=[Product.WEATHER_STATION],
-    ),
-    "air_pressure": Variable(
-        long_name="Air pressure",
-        units="Pa",
-        standard_name="air_pressure",
-        required=[Product.WEATHER_STATION],
     ),
     "rainfall_amount": Variable(
         long_name="Rainfall amount",
         units="m",
         standard_name="thickness_of_rainfall_amount",
-        required=[Product.WEATHER_STATION],
+        comment="Cumulated precipitation since 00:00 UTC",
+        required=[Product.RAIN_GAUGE],
+    ),
+    "precipitation_rate": Variable(
+        long_name="Precipitation rate",
+        units="m s-1",
+        standard_name="lwe_precipitation_rate",
+    ),
+    "precipitation_amount": Variable(
+        long_name="Precipitation amount",
+        units="m",
+        standard_name="lwe_thickness_of_precipitation_amount",
+        comment="Cumulated precipitation since 00:00 UTC",
+    ),
+    "snowfall_amount": Variable(
+        long_name="Snowfall amount",
+        units="m",
+        standard_name="thickness_of_snowfall_amount",
+        comment="Cumulated snow since 00:00 UTC",
     ),
     # ------------------------------------
     # Required in RAIN RADAR Level 1b file
@@ -179,50 +196,96 @@ VARIABLES = {
     "lidar_wavelength": Variable(
         long_name="Laser wavelength",
         units="nm",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "insect_prob": Variable(
         long_name="Insect probability",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "uwind": Variable(
         long_name="Zonal wind",
         units="m s-1",
-        required=[Product.CATEGORIZE],
+        required=[
+            Product.MODEL,
+            Product.CATEGORIZE,
+            Product.CATEGORIZE_VOODOO,
+            Product.DOPPLER_LIDAR_WIND,
+        ],
+    ),
+    "uwind_raw": Variable(
+        long_name="Non-screened zonal wind",
+        units="m s-1",
     ),
     "vwind": Variable(
         long_name="Meridional wind",
         units="m s-1",
-        required=[Product.CATEGORIZE],
+        required=[
+            Product.MODEL,
+            Product.CATEGORIZE,
+            Product.CATEGORIZE_VOODOO,
+            Product.DOPPLER_LIDAR_WIND,
+        ],
+    ),
+    "vwind_raw": Variable(
+        long_name="Non-screened meridional wind",
+        units="m s-1",
+    ),
+    "wwind": Variable(
+        long_name="Vertical wind",
+        units="m s-1",
+    ),
+    "azimuth_offset": Variable(
+        long_name="Azimuth offset of the instrument (positive clockwise from north)",
+        units="degrees",
+    ),
+    "zenith_offset": Variable(
+        long_name="Zenith offset of the instrument",
+        units="degrees",
     ),
     "q": Variable(
         long_name="Specific humidity",
-        required=[Product.CATEGORIZE],
+        units="1",
+        required=[Product.MODEL, Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
+    ),
+    "rh": Variable(
+        long_name="Relative humidity",
+        units="1",
+        required=[Product.MODEL],
     ),
     "Tw": Variable(
         long_name="Wet-bulb temperature",
         units="K",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "category_bits": Variable(
         long_name="Target categorization bits",
         dtype=Dtype.INT,
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "radar_liquid_atten": Variable(
         long_name="Two-way radar attenuation due to liquid water",
         units="dB",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "radar_gas_atten": Variable(
         long_name="Two-way radar attenuation due to atmospheric gases",
         units="dB",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
+    ),
+    "radar_rain_atten": Variable(
+        long_name="Two-way radar attenuation due to rain",
+        units="dB",
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
+    ),
+    "radar_melting_atten": Variable(
+        long_name="Two-way radar attenuation due to melting ice",
+        units="dB",
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "quality_bits": Variable(
         long_name="Data quality bits",
         dtype=Dtype.INT,
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "beta": Variable(
         long_name="Attenuated backscatter coefficient",
@@ -230,86 +293,87 @@ VARIABLES = {
         # Lidar has its own beta test
         required=[
             Product.CATEGORIZE,
+            Product.CATEGORIZE_VOODOO,
             Product.DOPPLER_LIDAR,
         ],
     ),
     "beta_error": Variable(
         long_name="Error in attenuated backscatter coefficient",
         units="dB",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "beta_bias": Variable(
         long_name="Bias in attenuated backscatter coefficient",
         units="dB",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "v_sigma": Variable(
         long_name="Standard deviation of mean Doppler velocity",
         units="m s-1",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "Z": Variable(
         long_name="Radar reflectivity factor",
         units="dBZ",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "Z_bias": Variable(
         long_name="Bias in radar reflectivity factor",
         units="dB",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "Z_error": Variable(
         long_name="Error in radar reflectivity factor",
         units="dB",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "Z_sensitivity": Variable(
         long_name="Minimum detectable radar reflectivity",
         units="dBZ",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "model_time": Variable(
         long_name="Model time UTC",
         units=time_units,
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     "model_height": Variable(
         long_name="Height of model variables above mean sea level",
         units="m",
-        required=[Product.CATEGORIZE],
+        required=[Product.CATEGORIZE, Product.CATEGORIZE_VOODOO],
     ),
     # ---------------------------------------
     # Required in CLASSIFICATION Level 2 file
     # ---------------------------------------
     "target_classification": Variable(
         long_name="Target classification",
-        required=[Product.CLASSIFICATION],
+        required=[Product.CLASSIFICATION, Product.CLASSIFICATION_VOODOO],
         dtype=Dtype.INT,
     ),
     "detection_status": Variable(
         long_name="Radar and lidar detection status",
-        required=[Product.CLASSIFICATION],
+        required=[Product.CLASSIFICATION, Product.CLASSIFICATION_VOODOO],
         dtype=Dtype.INT,
     ),
     "cloud_base_height_amsl": Variable(
         long_name="Height of cloud base above mean sea level",
         units="m",
-        required=[Product.CLASSIFICATION],
+        required=[Product.CLASSIFICATION, Product.CLASSIFICATION_VOODOO],
     ),
     "cloud_top_height_amsl": Variable(
         long_name="Height of cloud top above mean sea level",
         units="m",
-        required=[Product.CLASSIFICATION],
+        required=[Product.CLASSIFICATION, Product.CLASSIFICATION_VOODOO],
     ),
     "cloud_base_height_agl": Variable(
         long_name="Height of cloud base above ground level",
         units="m",
-        required=[Product.CLASSIFICATION],
+        required=[Product.CLASSIFICATION, Product.CLASSIFICATION_VOODOO],
     ),
     "cloud_top_height_agl": Variable(
         long_name="Height of cloud top above ground level",
         units="m",
-        required=[Product.CLASSIFICATION],
+        required=[Product.CLASSIFICATION, Product.CLASSIFICATION_VOODOO],
     ),
     # ----------------------------
     # Required in LWC Level 2 file
@@ -355,16 +419,16 @@ VARIABLES = {
         units="kg m-3",
         required=[Product.IWC],
     ),
-    "iwc_inc_rain": Variable(
-        long_name="Ice water content including rain",
-        units="kg m-3",
-        required=[Product.IWC],
-    ),
     "iwc_retrieval_status": Variable(
         long_name="Ice water content retrieval status",
         dtype=Dtype.INT,
         required=[Product.IWC],
     ),
+    # Deprecated:
+    "iwc_inc_rain": Variable(
+        long_name="Ice water content including rain",
+        units="kg m-3",
+    ),
     # --------------------------------
     # Required in DRIZZLE Level 2 file
     # --------------------------------
@@ -484,11 +548,6 @@ VARIABLES = {
         units="m",
         required=[Product.IER],
     ),
-    "ier_inc_rain": Variable(
-        long_name="Ice effective radius including rain",
-        units="m",
-        required=[Product.IER],
-    ),
     "ier_error": Variable(
         long_name="Random error in ice effective radius",
         units="m",
@@ -499,6 +558,11 @@ VARIABLES = {
         dtype=Dtype.INT,
         required=[Product.IER],
     ),
+    # Deprecated:
+    "ier_inc_rain": Variable(
+        long_name="Ice effective radius including rain",
+        units="m",
+    ),
     # ----------------------------
     # Required in DER Level 2 file
     # ----------------------------
@@ -567,7 +631,7 @@ VARIABLES = {
         required=[Product.MWR_L1C],
     ),
     "tb_spectrum": Variable(
-        long_name="Brightness temperature spectrum",
+        long_name="Retrieved brightness temperature spectrum",
         units="K",
     ),
     "receiver_nb": Variable(
@@ -576,7 +640,7 @@ VARIABLES = {
         required=[Product.MWR_L1C],
     ),
     "met_quality_flag": Variable(
-        long_name="Meterological data quality flag",
+        long_name="Meteorological data quality flag",
         dtype=Dtype.INT,
         required=[Product.MWR_L1C],
     ),
@@ -648,37 +712,37 @@ VARIABLES = {
     # Required in MWR SINGLE file
     # ---------------------------
     "iwv_quality_flag": Variable(
-        long_name="Quality flag",
+        long_name="Integrated water vapour quality flag",
         units="1",
         required=[Product.MWR_SINGLE],
         dtype=Dtype.INT,
     ),
     "iwv_quality_flag_status": Variable(
-        long_name="Quality flag status",
+        long_name="Integrated water vapour quality flag status",
         units="1",
         required=[Product.MWR_SINGLE],
         dtype=Dtype.INT,
     ),
     "absolute_humidity_quality_flag": Variable(
-        long_name="Quality flag",
+        long_name="Absolute humidity quality flag",
         units="1",
         required=[Product.MWR_SINGLE],
         dtype=Dtype.INT,
     ),
     "absolute_humidity_quality_flag_status": Variable(
-        long_name="Quality flag status",
+        long_name="Absolute humidity quality flag status",
         units="1",
         required=[Product.MWR_SINGLE],
         dtype=Dtype.INT,
     ),
     "lwp_quality_flag": Variable(
-        long_name="Quality flag",
+        long_name="Liquid water path quality flag",
         units="1",
         required=[Product.MWR_SINGLE],
         dtype=Dtype.INT,
     ),
     "lwp_quality_flag_status": Variable(
-        long_name="Quality flag status",
+        long_name="Liquid water path quality flag status",
         units="1",
         required=[Product.MWR_SINGLE],
         dtype=Dtype.INT,
@@ -699,45 +763,151 @@ VARIABLES = {
     "potential_temperature": Variable(
         long_name="Potential temperature",
         units="K",
-        required=[
-            Product.MWR_SINGLE,
-            Product.MWR_MULTI,
-        ],
+        required=[Product.MWR_MULTI],
     ),
     "equivalent_potential_temperature": Variable(
         long_name="Equivalent potential temperature",
         units="K",
-        required=[
-            Product.MWR_SINGLE,
-            Product.MWR_MULTI,
-        ],
+        required=[Product.MWR_MULTI],
     ),
     "relative_humidity": Variable(
         long_name="Relative humidity",
         standard_name="relative_humidity",
-        required=[
-            Product.MWR_SINGLE,
-            Product.MWR_MULTI,
-        ],
+        required=[Product.MWR_MULTI],
     ),
     "temperature_quality_flag": Variable(
-        long_name="Quality flag",
+        long_name="Temperature quality flag",
         units="1",
-        required=[
-            Product.MWR_SINGLE,
-            Product.MWR_MULTI,
-        ],
+        required=[Product.MWR_MULTI],
         dtype=Dtype.INT,
     ),
     "temperature_quality_flag_status": Variable(
-        long_name="Quality flag status",
+        long_name="Temperature quality flag status",
         units="1",
-        required=[
-            Product.MWR_SINGLE,
-            Product.MWR_MULTI,
-        ],
+        required=[Product.MWR_MULTI],
         dtype=Dtype.INT,
     ),
+    # ---------------------------------------
+    # Required in cpr-simulation Level 3 file
+    # ---------------------------------------
+    "sat_ifov": Variable(
+        long_name="Satellite instantaneous field of view",
+        units="m",
+        required=[Product.CPR],
+    ),
+    "sat_range_resolution": Variable(
+        long_name="Satellite range resolution",
+        units="m",
+        required=[Product.CPR],
+    ),
+    "sat_along_track_resolution": Variable(
+        long_name="Satellite along-track resolution",
+        units="m",
+        required=[Product.CPR],
+    ),
+    "ze": Variable(
+        long_name="Radar reflectivity factor of input",
+        units="dBZ",
+        required=[Product.CPR],
+    ),
+    "vm": Variable(
+        long_name="Mean Doppler velocity of input",
+        units="m s-1",
+        required=[Product.CPR],
+    ),
+    "ze_sat": Variable(
+        long_name="Convolved and integrated radar reflectivity factor",
+        units="dBZ",
+        required=[Product.CPR],
+    ),
+    "vm_sat": Variable(
+        long_name="Convolved and integrated mean Doppler velocity",
+        units="m s-1",
+        required=[Product.CPR],
+    ),
+    "vm_sat_vel": Variable(
+        long_name=(
+            "Convolved and integrated mean Doppler velocity with satellite motion error"
+        ),
+        units="m s-1",
+        required=[Product.CPR],
+    ),
+    "vm_sat_noise": Variable(
+        long_name=(
+            "Convolved and integrated mean Doppler velocity "
+            "with noise and satellite motion error"
+        ),
+        units="m s-1",
+        required=[Product.CPR],
+    ),
+    "ze_sat_noise": Variable(
+        long_name="Convolved and integrated radar reflectivity factor with noise",
+        units="dBZ",
+        required=[Product.CPR],
+    ),
+    "vm_sat_folded": Variable(
+        long_name="Doppler velocity with noise, satellite motion error, and folding",
+        units="m s-1",
+        required=[Product.CPR],
+    ),
+    "nubf": Variable(
+        long_name="Non-uniform beam filling",
+        units="dBZ",
+        required=[Product.CPR],
+    ),
+    "nubf_flag": Variable(
+        long_name="Non-uniform beam filling flag",
+        dtype=Dtype.INT,
+        required=[Product.CPR],
+    ),
+    "ms_flag": Variable(
+        long_name="Multiple scattering flag",
+        dtype=Dtype.INT,
+        required=[Product.CPR],
+    ),
+    "folding_flag": Variable(
+        long_name="Folding flag",
+        dtype=Dtype.INT,
+        required=[Product.CPR],
+    ),
+    "along_track_sat": Variable(
+        long_name="Along-track distance at satellite resolution",
+        units="m",
+        required=[Product.CPR],
+    ),
+    "along_track": Variable(
+        long_name="Along-track distance",
+        units="m",
+        required=[Product.CPR],
+    ),
+    "mean_wind": Variable(
+        long_name="Mean horizontal wind",
+        units="m s-1",
+        required=[Product.CPR],
+    ),
+    "height_sat": Variable(
+        long_name="Height above mean sea level at satellite resolution",
+        units="m",
+        dtype=Dtype.FLOAT,
+        required=[Product.CPR],
+    ),
+    # ----------------------------------------
+    # Required in Epsilon (lidar)
+    # ----------------------------------------
+    "epsilon": Variable(
+        long_name="Dissipation rate of turbulent kinetic energy",
+        units="m2 s-3",
+        required=[
+            Product.EPSILON_LIDAR,
+        ],
+    ),
+    "rolling_window_period": Variable(
+        long_name="Rolling window period",
+        units="s",
+        required=[
+            Product.EPSILON_LIDAR,
+        ],
+    ),
     # -------------------------
     # Required in several files
     # -------------------------
@@ -746,8 +916,8 @@ VARIABLES = {
         units="m s-1",
         standard_name="rainfall_rate",
         required=[
+            Product.RAIN_GAUGE,
             Product.DISDROMETER,
-            Product.WEATHER_STATION,
             Product.RAIN_RADAR,
         ],
     ),
@@ -765,6 +935,7 @@ VARIABLES = {
         required=[
             Product.RADAR,
             Product.CATEGORIZE,
+            Product.CATEGORIZE_VOODOO,
             Product.DOPPLER_LIDAR,
             Product.RAIN_RADAR,
         ],
@@ -775,6 +946,7 @@ VARIABLES = {
         required=[
             Product.MODEL,
             Product.CATEGORIZE,
+            Product.CATEGORIZE_VOODOO,
             Product.MWR_MULTI,
         ],
     ),
@@ -784,6 +956,7 @@ VARIABLES = {
         required=[
             Product.MODEL,
             Product.CATEGORIZE,
+            Product.CATEGORIZE_VOODOO,
         ],
     ),
     "lwp": Variable(
@@ -792,7 +965,6 @@ VARIABLES = {
         standard_name="atmosphere_cloud_liquid_water_content",
         required=[
             Product.MWR,
-            Product.CATEGORIZE,
             Product.LWC,
             Product.MWR_SINGLE,
         ],
@@ -800,10 +972,7 @@ VARIABLES = {
     "lwp_error": Variable(
         long_name="Error in liquid water path",
         units="kg m-2",
-        required=[
-            Product.CATEGORIZE,
-            Product.LWC,
-        ],
+        required=[Product.LWC],
     ),
     "height": Variable(
         long_name="Height above mean sea level",
@@ -814,6 +983,7 @@ VARIABLES = {
             Product.MWR,
             Product.DISDROMETER,
             Product.WEATHER_STATION,
+            Product.RAIN_GAUGE,
             Product.MWR_L1C,
             Product.L3_CF,
             Product.L3_IWC,
@@ -856,15 +1026,21 @@ VARIABLES = {
         long_name="Radar range offset",
         units="m",
     ),
+    "r_accum_RT": Variable(
+        long_name="Real time accumulated rainfall",
+        units="m",
+    ),
+    "r_accum_NRT": Variable(
+        long_name="Near real time accumulated rainfall",
+        units="m",
+    ),
     "rainfall_rate_1min_total": Variable(
         long_name="Total precipitation rate",
         units="m s-1",
-        dtype=Dtype.INT,
     ),
     "rainfall_rate_1min_solid": Variable(
         long_name="Solid precipitation rate",
         units="m s-1",
-        dtype=Dtype.INT,
     ),
     "snowfall_rate": Variable(
         long_name="Snowfall rate",
@@ -926,7 +1102,6 @@ VARIABLES = {
     "V_sensor_supply": Variable(
         long_name="Sensor supply voltage",
         units="V",
-        dtype=Dtype.INT,
     ),
     "state_sensor": Variable(
         long_name="State of the sensor",
@@ -971,6 +1146,14 @@ VARIABLES = {
         long_name="Attenuated backscatter coefficient",
         units="sr-1 m-1",
     ),
+    "beta_cross_raw": Variable(
+        long_name="Attenuated backscatter coefficient for the cross-polarised signal",
+        units="sr-1 m-1",
+    ),
+    "beta_cross": Variable(
+        long_name="Attenuated backscatter coefficient for the cross-polarised signal",
+        units="sr-1 m-1",
+    ),
     "iwv": Variable(
         long_name="Integrated water vapour",
         units="kg m-2",
@@ -1101,6 +1284,11 @@ VARIABLES = {
     "status_flag": Variable(
         long_name="Status flag for heater and blower",
     ),
+    "air_pressure": Variable(
+        long_name="Air pressure",
+        units="Pa",
+        standard_name="air_pressure",
+    ),
     "wind_speed": Variable(
         long_name="Wind speed",
         standard_name="wind_speed",
@@ -1127,12 +1315,6 @@ VARIABLES = {
         long_name="Model level",
         units=None,
         dtype=Dtype.SHORT,
-        required=[
-            Product.MODEL,
-            Product.L3_CF,
-            Product.L3_IWC,
-            Product.L3_LWC,
-        ],
     ),
     "flux_level": Variable(
         long_name="Model flux level",
@@ -1313,7 +1495,6 @@ VARIABLES = {
     "I_mean_laser": Variable(
         long_name="Mean value of laser current",
         units="mA",
-        dtype=Dtype.INT,
     ),
     "V_control": Variable(
         long_name="Control voltage",
@@ -1338,7 +1519,6 @@ VARIABLES = {
     "maximum_hail_diameter": Variable(
         long_name="Maximum hail diameter",
         units="mm",
-        dtype=Dtype.INT,
     ),
     # rv-polarstern specific
     "beta_1064_raw": Variable(
@@ -1429,8 +1609,9 @@ VARIABLES = {
     "snr_355_nr": Variable(
         long_name="Signal-to-Noise Ratio (355 nm near range)",
     ),
+    # rv-polarstern categorize specific
     "lidar_depolarisation": Variable(
-        long_name="Calibrated volume depolarisation (532 nm)"  # in rv-polarstern categorize files
+        long_name="Calibrated volume depolarisation (532 nm)"
     ),
     "beamwidthV": Variable(
         long_name="Vertical angular beamwidth",
@@ -1441,4 +1622,111 @@ VARIABLES = {
         units="degree",
     ),
     "rain_detected": Variable(long_name="Rain detected", dtype=Dtype.INT),
+    "air_pressure_quality_flag": Variable(
+        long_name="Air pressure quality flag",
+        dtype=Dtype.BYTE,
+    ),
+    "air_temperature_quality_flag": Variable(
+        long_name="Air temperature quality flag",
+        dtype=Dtype.BYTE,
+    ),
+    "relative_humidity_quality_flag": Variable(
+        long_name="Relative humidity quality flag",
+        dtype=Dtype.BYTE,
+    ),
+    "wind_speed_quality_flag": Variable(
+        long_name="Wind speed quality flag",
+        dtype=Dtype.BYTE,
+    ),
+    "wind_direction_quality_flag": Variable(
+        long_name="Wind direction quality flag",
+        dtype=Dtype.BYTE,
+    ),
+    "rainfall_rate_quality_flag": Variable(
+        long_name="Rainfall rate quality flag",
+        dtype=Dtype.BYTE,
+    ),
+    "dew_point_temperature": Variable(
+        long_name="Dew point temperature",
+        dtype=Dtype.FLOAT,
+        units="K",
+    ),
+    "dew_point_temperature_quality_flag": Variable(
+        long_name="Dew point temperature quality flag",
+        dtype=Dtype.BYTE,
+    ),
+    "wind_speed_gust": Variable(
+        long_name="Wind speed gust",
+        dtype=Dtype.FLOAT,
+        units="m s-1",
+    ),
+    "wind_speed_gust_quality_flag": Variable(
+        long_name="Wind speed gust quality flag",
+        dtype=Dtype.BYTE,
+    ),
+    "visibility_quality_flag": Variable(
+        long_name="Visibility quality flag",
+        dtype=Dtype.BYTE,
+    ),
+    "elevation_angle": Variable(
+        long_name="Sensor elevation angle",
+        units="degree",
+    ),
+    "ir_elevation_angle": Variable(
+        long_name="Infrared sensor elevation angle",
+        units="degree",
+    ),
+    "liquid_prob": Variable(
+        long_name="Liquid probability",
+        required=[Product.CATEGORIZE_VOODOO],
+    ),
+    "polariser_bleed_through": Variable(
+        long_name="Polariser bleed-through",
+    ),
+    "pulses_per_ray": Variable(long_name="Pulses per ray", units="1", dtype=Dtype.INT),
+    "ray_accumulation_time": Variable(
+        long_name="Ray accumulation time", units="s", dtype=Dtype.FLOAT
+    ),
+    "lifted_index": Variable(
+        long_name="Lifted index",
+    ),
+    "ko_index": Variable(
+        long_name="KO index",
+    ),
+    "total_totals": Variable(
+        long_name="Total totals index",
+    ),
+    "k_index": Variable(
+        long_name="K index",
+    ),
+    "showalter_index": Variable(
+        long_name="Showalter index",
+    ),
+    "cape": Variable(
+        long_name="Convective available potential energy",
+    ),
+    "stability_quality_flag": Variable(
+        long_name="Quality flag for stability products",
+        dtype=Dtype.INT,
+    ),
+    "stability_quality_flag_status": Variable(
+        long_name="Quality flag status for stability products",
+        dtype=Dtype.INT,
+    ),
+    "visibility": Variable(
+        long_name="Meteorological optical range (MOR) visibility",
+        units="m",
+        standard_name="visibility_in_air",
+        dtype=Dtype.INT,
+    ),
+    "snr_limit": Variable(
+        long_name="SNR limit",
+        units="dB",
+        comment="SNR threshold used in data screening.",
+        dtype=Dtype.FLOAT,
+    ),
+    "correction_bits": Variable(
+        long_name="Correction bits",
+        dtype=Dtype.INT,
+    ),
 }
